
float4x4 matWorldViewProj	: WORLDVIEWPROJ;	//our world view projection matrix
float4x4 matWorldI          : WORLDIT;
float4x4 matWorld           : WORLD;

float4   EyePos				= float4(0.0f, 0.0f, 0.0f, 0.0f);


//------------------------------------

texture Texture0;

sampler2D texSampler0 : TEXUNIT0 = sampler_state
{
	Texture	  = (Texture0);
    MIPFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
};


//application to vertex structure
struct vin
{ 
    float4 position   : POSITION0;
    float3 normal	  : NORMAL;
    float2 tex0       : TEXCOORD0;
};
 
//vertex to pixel shader structure
struct vout
{
    float4 position	   : POSITION0;
    float4 colour      : COLOR0;
    float2 tex0        : TEXCOORD0;
    float3 tex1        : TEXCOORD1;
};

//VERTEX SHADER
vout vs( in vin IN )
{
	//init our output data to all zeros
	vout OUT = (vout)0;

    //getting to position to object space
    OUT.position    = mul(IN.position, matWorldViewProj);
	
    //Move Eye Position to world Space
    float4 worldEyePos = mul(EyePos, matWorld);
    
    //Move our object into world space
    float4 worldVertPos = mul(IN.position, matWorld);
    
    //Create our eye diretion vector
    float4 E = normalize(worldEyePos - worldVertPos);
    
    //Convert our vertex normal to world space
    float4 N = mul(IN.normal, matWorld); 

    // Compute N-dot-E
    float n_dot_e = dot(N, E);
    
    // Clamp to [0..1]
	n_dot_e = clamp( n_dot_e, 0, 1 );
   
	//pass texture data on to pixel stage
	OUT.tex0.x = n_dot_e;
	OUT.tex0.y = 0;

	//Use colour output to pass the alpha value
	OUT.colour.a = 1-n_dot_e;
	
	return OUT;
}

float4 ps( in vout IN ) : COLOR
{
	float4 finalcolour = tex2D(texSampler0, IN.tex0);
	
	//use colour alpha value
	finalcolour.a = IN.colour.a;
	
    //return final pixel colour
	return finalcolour;
}


//------------------------------------
technique Hologram
{
	pass p0 
	{		
		
		vertexshader = compile vs_1_1 vs();
		
		pixelshader  = compile ps_1_4 ps();
		
		//---------------------------------------------------------------------------//
		// Final = Dest * DestBlend + Src * SrcBlend, where dest is the pixel 
		// on the screen and src is the texel color.
		// blends in D3D are: SRCCOLOR, INVSRCCOLOR, SRCALPHA, INVSRCALPHA, 
		// DESTCOLOR, INVDESTCOLOR, DESTALPHA, INVDESTALPHA, ONE, and ZERO.

		// Final = Dest * DestBlend + Src * SrcBlend
		
		SrcBlend  = SRCALPHA;
		DestBlend = INVSRCALPHA;

		
		AlphaBlendEnable = TRUE;
	
		Texture[0]		= <Texture0>;
		MinFilter[0]	= Point;
		MagFilter[0]	= Point;
		MipFilter[0]	= Point;
	}
}


